library(easyNCDF)

# Create directory to store example generated files
tmp_dir <- tempdir()
test_dir <- paste0(tmp_dir, '/testthat')
dir.create(test_dir)

# Helper function to drop dims without removing dimnames
drop2 <- function(x) {
  if (!is.null(dim(x))) {
    dims_to_drop <- which(dim(x) == 1)
    if (length(dims_to_drop) > 0) {
      if (length(dims_to_drop) == length(dim(x))) {
        dim(x) <- NULL
      } else {
        dim(x) <- dim(x)[-dims_to_drop]
      }
    }
  }
  x
}

# Helper function to remove attributes
unattr <- function(x) {
  attributes(x, 'variables') <- NULL
  x
}

################################################################################
################################################################################
################################################################################
context("Retrieving and aligning multidimensional data sets.")

################################################################################
################################################################################
test_that("Call with no pattern dimension crashes.", {
################################################################################
  expect_error(
    Start(), 
    "At least one pattern dim must be specified."
  )
################################################################################
  expect_error(
    Start(1), 
    "At least one pattern dim must be specified."
  )
################################################################################
  expect_error(
    Start('a'), 
    "At least one pattern dim must be specified."
  )
################################################################################
  expect_error(
    Start('all'), 
    "At least one pattern dim must be specified."
  )
})

################################################################################
################################################################################
test_that("Call with unexisting file crashes.", {
################################################################################
  expect_error(
    Start(dataset = list(
                      list(path = './non_existing_file.nc',
                           name = 'Dataset A')
                    ),
          pattern_dims = 'dataset'),
    "No data files found for any of the specified datasets."
  )
})

################################################################################
################################################################################
  # We need to put as.numeric, because loaded data with Start has to go through
  # a big.matrix which is always of type 'numeric', regardless of the type of 
  # the data. Since the big.matrix is created before reading any data, it is
  # not possible to create it with a special type depending on the type of 
  # the data.
sin_var__mul_val__mul_dim <- array(as.numeric(1:9), c(longitude = 9, latitude = 9))
sin_var__mul_val__mul_dim__path <- paste0(test_dir, '/sin_var__mul_val__mul_dim.nc')
ArrayToNc(list(temperature = sin_var__mul_val__mul_dim), sin_var__mul_val__mul_dim__path)
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and no inner selectors specified crashes. One core only.", {
################################################################################
  expect_error(
    Start(dataset = list(
                      list(path = sin_var__mul_val__mul_dim__path,
                           name = 'Dataset A')
                    ),
          pattern_dims = 'dataset',
          metadata_dims = NA,
          num_procs = 1),
    regexp = "Unexpected extra dimensions \\(of length > 1\\) in the file.*"
  )
})

################################################################################
################################################################################
sin_var__sin_val__mul_dim <- array(5, c(longitude = 1, latitude = 1))
sin_var__sin_val__mul_dim__path <- paste0(test_dir, '/sin_var__sin_val__mul_dim.nc')
ArrayToNc(list(temperature = sin_var__sin_val__mul_dim), sin_var__sin_val__mul_dim__path)
test_that("Call with existing file with single variable, single value and inner dimensions of length = 1 and no inner selectors specified works. One core only.", {
################################################################################
  data <- Start(dataset = list(
                            list(path = sin_var__sin_val__mul_dim__path,
                                 name = 'Dataset A')
                            ),
                pattern_dims = 'dataset',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), drop2(sin_var__sin_val__mul_dim))
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only.", {
################################################################################
  data <- Start(dataset = list(
                            list(path = sin_var__mul_val__mul_dim__path,
                                 name = 'Dataset A')
                            ),
                pattern_dims = 'dataset',
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), sin_var__mul_val__mul_dim)
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only. Dataset specification without name.", {
################################################################################
  data <- Start(dataset = list(
                            list(path = sin_var__mul_val__mul_dim__path)
                            ),
                pattern_dims = 'dataset',
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), sin_var__mul_val__mul_dim)
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only. Dataset specification as single list.", {
################################################################################
  data <- Start(dataset = list(path = sin_var__mul_val__mul_dim__path),
                pattern_dims = 'dataset',
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), sin_var__mul_val__mul_dim)
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only. Dataset specification as single path pattern.", {
################################################################################
  data <- Start(dataset = sin_var__mul_val__mul_dim__path,
                pattern_dims = 'dataset',
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), sin_var__mul_val__mul_dim)
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only. Multiple datasets specified as multiple path patterns.", {
################################################################################
  data <- Start(dataset = c(sin_var__mul_val__mul_dim__path,
                            sin_var__mul_val__mul_dim__path),
                pattern_dims = 'dataset',
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(Subset(data$Data, 'dataset', 1, drop = 'selected'), 
                   sin_var__mul_val__mul_dim)
})

################################################################################
################################################################################
test_that("Call with existing file with single variable, multiple values and inner dimensions of length > 1, and all inner selectors specified works. One core only. Dataset specified as path pattern. No pattern dim specified.", {
################################################################################
  data <- Start(dataset = sin_var__mul_val__mul_dim__path,
                longitude = 'all',
                latitude = 'all',
                metadata_dims = NA,
                num_procs = 1)
  expect_identical(drop2(data$Data), sin_var__mul_val__mul_dim)
})
